<?php

  /*
    Auth - class to run http authentication in a directory using
           cookies and sessions
    author: Antonio Bonifati - http://go.to/ninuzzo/
  */

  class Auth
  { var
      # change default options here
      $options = array(
        /* name of the session variable used to identify user. It can be an
           array of accepted user names and it will be changed to the
           appropriate string value by the constructor or by the first call to
           logged(). Defaults to a single value 'username' */
        'userIdVarName' => 'username',

        'URI' => array(
	  /* absolute URI to which the method loginOk() redirects
             (defaults to 'current script directory/loginOk.php'; see constructor) */
          'loginOk' => '',

          /* absolute URI to which the loginFailed() method redirects
             (default to 'current script directory/loginFailed.html'; see constructor) */
          'loginFailed' => '',

          /* cookie path (defaults to the current script directory; see constructor) */
          'cookiePath' => ''
        ),

        # name of the cookie
        # this has been changed from default PHPSESSID to SESSID to not reveal PHP use
        'cookieName' => 'SESSID',
      ),

      /* it is automatically set to TRUE if the server is secure
         (supports HTTPS), FALSE otherwise */
      $serverSecure;

    # constructor
    function Auth($options=array())
    { $this->options = array_merge($this->options, $options);
      if (!isset($this->options['URI']['cookiePath']) || $this->options['URI']['cookiePath']=='')
        $this->options['URI']['cookiePath']=dirname($_SERVER['PHP_SELF']);
      if (!isset($this->options['URI']['loginOk']) || $this->options['URI']['loginOk']=='')
        $this->options['URI']['loginOk']=dirname($_SERVER['PHP_SELF']).'/loginOk.php';
      if (!isset($this->options['URI']['loginFailed']) || $this->options['URI']['loginFailed']=='')
        $this->options['URI']['loginFailed']=dirname($_SERVER['PHP_SELF']).'/loginFailed.html';

      if (gettype($this->options['userIdVarName'])=='array')
      { # determine the username passed with the POST method if any
        foreach ($this->options['userIdVarName'] as $uIdVN)
          if (isset($_POST[$uIdVN]) && $_POST[$uIdVN]!='')
          { $this->options['userIdVarName'] = $uIdVN;
            break;
          }
      }

      $this->serverSecure = (
        isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on' ? TRUE : FALSE
      );
    }

    # gets the contents of the session variable used to identify the user
    function getUserId()
    { return $_SESSION[$this->options['userIdVarName']];
    }

    function startSession()
    { session_name($this->options['cookieName']);
      session_set_cookie_params(
        0,					# lifetime
        $this->options['URI']['cookiePath'],	# path
        $_SERVER['SERVER_NAME'],		# domain
        $this->serverSecure			# secure
      );
      session_start();
    }

    /* creates a new session for the user and registers him/her login
       must be called before any output is sent to the browser */
    function loginOk($userId, $sessVars=array())
    { $this->startSession();

      # register user id in session variable
      $_SESSION[$this->options['userIdVarName']]=$userId;
      $_SESSION = array_merge($_SESSION, $sessVars);

      $this->redirect($this->options['URI']['loginOk']);
    }

    /* redirects to the login failed page
       must be called before any output is sent to the browser */
    function loginFailed()
    { $this->redirect($this->options['URI']['loginFailed']);
    }

    /* logs the user out
       must be called before any output is sent to the browser */
    function logout()
    { $this->startSession();

      /* delete cookie on the client side
         this is optional */
      setcookie(
        $this->options['cookieName'],
        '',
        0,
        $this->options['URI']['cookiePath'],
        $_SERVER['SERVER_NAME'],
        $this->serverSecure
      );

      /* delete session on server
         this is mandatory */
      session_destroy();
    }

    /* redirect browser to the specified relative uri
       must be called before any output is sent to the browser */
    function redirect($uri='')
    { if ($uri=='')
        $uri=dirname($_SERVER['PHP_SELF']);

      header(
        'Location: ' . ($this->serverSecure ? 'https' : 'http').
        "://$_SERVER[HTTP_HOST]$uri"
      );
      exit;
    }

    # check if the user is logged in
    function logged()
    { $this->startSession();

      if (gettype($this->options['userIdVarName'])=='array')
      { # determine the username
        foreach ($this->options['userIdVarName'] as $uIdVN)
          if (isset($_SESSION[$uIdVN]))
          { $this->options['userIdVarName'] = $uIdVN;
            return true;
          }
        return false;
      }

      return isset($_SESSION[$this->options['userIdVarName']]);
    }
  }

?>
